library circle_list;

import 'dart:math';

import 'package:circle_list/radial_drag_gesture_detector.dart';
export 'package:circle_list/radial_drag_gesture_detector.dart';
import 'package:flutter/material.dart';

class CircleList extends StatefulWidget {
  final double innerRadius;
  final double outerRadius;
  final double childrenPadding;
  final double initialAngle;
  final Color outCircleColor;
  final Color innerCircleColor;
  final Offset origin;
  final List<Widget> children;
  final RadialDragStart onDragStart;
  final RadialDragUpdate onDragUpdate;
  final RadialDragEnd onDragEnd;

  CircleList(
      {this.innerRadius,
        this.outerRadius,
        this.childrenPadding = 10,
        this.initialAngle = 0,
        this.outCircleColor,
        this.innerCircleColor,
        this.origin,
        @required this.children,
        this.onDragStart,
        this.onDragUpdate,
        this.onDragEnd})
      : assert(children != null);

  @override
  _CircleListState createState() => _CircleListState();
}

class _CircleListState extends State<CircleList> {
  DragModel dragModel = DragModel();

  @override
  Widget build(BuildContext context) {
    final size = MediaQuery.of(context).size;
    final outCircleDiameter = min(size.width, size.height);
    final double outerRadius = widget.outerRadius ?? outCircleDiameter / 2;
    final double innerRadius = widget.innerRadius ?? outerRadius / 2;
    final double betweenRadius = (outerRadius + innerRadius) / 2;
    final Offset origin = widget.origin ?? Offset(0, -outerRadius);

    return Stack(
      children: <Widget>[
        Positioned(
          left: origin.dx,
          bottom: origin.dy,
          child: Container(
              width: outerRadius * 2,
              height: outerRadius * 2,
              decoration: BoxDecoration(
                  color: widget.innerCircleColor ?? Colors.transparent,
                  shape: BoxShape.circle,
                  border: Border.all(
                      color: widget.outCircleColor ?? Colors.transparent,
                      width: outerRadius - innerRadius))),
        ),
        Positioned(
          left: origin.dx,
          bottom: origin.dy,
          child: Container(
            width: outerRadius * 2,
            height: outerRadius * 2,
            color: Colors.transparent,
            child: RadialDragGestureDetector(
              onRadialDragUpdate: (PolarCoord updateCoord) {
                if(widget.onDragUpdate != null) {
                  widget.onDragUpdate(updateCoord);
                }
                setState(() {
                  dragModel.getAngleDiff(updateCoord);
                });
              },
              onRadialDragStart: (PolarCoord startCoord) {
                if(widget.onDragStart != null){
                  widget.onDragStart(startCoord);
                }
                setState(() {
                  dragModel.start = startCoord;
                });
              },
              onRadialDragEnd: () {
                if(widget.onDragEnd != null){
                  widget.onDragEnd();
                }
                dragModel.end = dragModel.start;
                dragModel.end.angle = dragModel.angleDiff;
              },
              child: Transform.rotate(
                angle: dragModel.angleDiff + widget.initialAngle,
                child: Stack(
                  children: List.generate(widget.children.length, (index) {
//                    debugPrint("当前的角度:${dragModel.dragAngle}");
                    final double childrenDiameter =
                        2 * pi * betweenRadius / 10 - widget.childrenPadding;
                    Offset childPoint = getChildPoint(
                        index, 10, betweenRadius, childrenDiameter);
                    return Positioned(
                      left: outerRadius + childPoint.dx,
                      top: outerRadius + childPoint.dy,
                      child: Transform.rotate(
                        angle: -(dragModel.angleDiff) + widget.initialAngle,
                        child: Container(
                            width: childrenDiameter,
                            height: childrenDiameter,
                            alignment: Alignment.center,
                            child: widget.children[index]),
                      ),
                    );
                  }),
                ),
              ),
            ),
          ),
        ),
      ],
    );
  }

  Offset getChildPoint(
      int index, int length, double betweenRadius, double childrenDiameter) {
    double angel = 2 * pi * (index / length);
    double x = cos(angel) * betweenRadius - childrenDiameter / 2;
    double y = sin(angel) * betweenRadius - childrenDiameter / 2;
    return Offset(x, y);
  }
}

class DragModel {
  PolarCoord start;
  PolarCoord end;
  double angleDiff = 0.0;

  double getAngleDiff(PolarCoord updatePolar) {
    if (start != null) {
      angleDiff = updatePolar.angle - start.angle;
      if (end != null) {
        angleDiff += end.angle;
      }
    }
    return angleDiff;
  }
}